#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma version=1.81
#pragma IndependentModule=OpusLoader
#pragma IgorVersion=7

#if IgorVersion() >= 8
#include "OpusBrowser", optional
#endif

// Project Updater header
static constant kProjectID=22397 // the project node on IgorExchange
static strconstant ksShortTitle="OpusLoader" // the project short title on IgorExchange

constant kPreferencesDialog = 1 // set to zero to avoid extended dialog

static strconstant ksPackageName = "OpusLoader"
static strconstant ksPrefsFileName = "acwOpusLoader.bin"
static constant kPrefsVersion = 100

//#define debug

// Loader for Bruker OPUS FTIR files

// Doesn't extract phase spectra. If you want to load them, let me know.

// 1.64 28/10/21 parameter blocks for both REF and SC are read correctly
// Instrument parameters are recorded in wavenotes.
// 1.62 26/03/21 write wavenotes to record acquisition parameters in unified style,
// using nested StringByKey
// 1.61 11/03/21 catch error when trying to read a corrupt (truncated) file.
// 1.51 12/31/18 save collection parameters in wavenotes of REF and SC
// 1.50 06/27/16 added option to include file extension in name, for Jed.
// 1.40 06/23/16 Bug fix. Turning on atmospheric compensation results in a
// file that has two blocks of absorbance data. quick fix: overwrite the
// first block with the second (only if the first was found, thus
// preserving the function of the overwrite setting).
// 1.30 11/03/11 no need for multiopenfiles xop anymore.
// 1.20 02/12/10 option to store waves in subfolders
// 1.10 02/09/10 loads single and reference channels and interferograms
// 1.00 02/03/10 rewrite of my older and more hacky version

menu "Load Waves"
	"-"
	"Load Bruker OPUS Files...", /Q, OpusLoader#LoadOpusGUI()
end

structure SpectrumTypeStrings
	char AB[6]
	char REF[6]
	char SC[6]
	char IFR[6]
	char IFS[6]
	char TR[6]
	char IMG[6]
	char reserved[18]
endstructure

structure PackagePrefs
	uint16 version // 2 bytes
	uint16 options // 2 bytes, 1 = subfolders, 2 = use extension, 4 = split matrix into spectra, 8 = plot fpa image, 16 = plot image
	uint16 loadoptions // 2 bytes, 1 = ABS, 2 = REF, 4 = SC, 8 = IFM, 16 = TR, 2^15 reserved
	char verbose
	char overwrite // 1 byte, 0 = rename, 1 = yes, 2 = ask
	STRUCT SpectrumTypeStrings prefix // 60 bytes
	STRUCT SpectrumTypeStrings suffix // 60 bytes
	char reserved[256 - 128]
endstructure

// set prefs structure to default values
function PrefsSetDefaults(STRUCT PackagePrefs &prefs)
	prefs.version     = kPrefsVersion
	prefs.options     = 3
	prefs.loadoptions = 15
	prefs.verbose     = 3
	prefs.overwrite   = 2
	prefs.prefix.AB   = ""
	prefs.prefix.REF  = "REF_"
	prefs.prefix.SC   = "SC_"
	prefs.prefix.IFR  = "IFR_"
	prefs.prefix.IFS  = "IFS_"
	prefs.prefix.TR   = "TR_"
	prefs.prefix.IMG  = "IMG_"
	prefs.suffix.AB   = ""
	prefs.suffix.REF  = ""
	prefs.suffix.SC   = ""
	prefs.suffix.IFR  = ""
	prefs.suffix.IFS  = ""
	prefs.suffix.TR   = ""
	prefs.suffix.IMG  = ""
	int i
	for(i=0;i<(256-128);i+=1)
		prefs.reserved[i] = 0
	endfor
end

function LoadPrefs(STRUCT PackagePrefs &prefs)
	LoadPackagePreferences/MIS=1 ksPackageName, ksPrefsFileName, 0, prefs
	if (V_flag!=0 || V_bytesRead==0 || prefs.version!=kPrefsVersion)
		PrefsSetDefaults(prefs)
		SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
	endif
end

function LoadOpusGUI()
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
					
	if (kPreferencesDialog)
			
		KillWindow/Z OpusLoaderPanel
			
		STRUCT point screenOrigin
		GetScreenOrigin(screenOrigin)
		
		int left = screenOrigin.h + 300
		int top = screenOrigin.v + 200
		int right = left + 250
		int bottom = top + 190

		NewPanel/K=2/N=OpusLoaderPanel/W=(left, top, right, bottom) as "Load OPUS Files"
		ModifyPanel/W=OpusLoaderPanel fixedsize=1
		// fix for negative coordinates bug
		#if (NumberByKey("Build", IgorInfo(0)) < 39580)
		variable cpu2p = ScreenResolution > 96 ? 1 : 72 / ScreenResolution
		MoveWindow/W=OpusLoaderPanel left*cpu2p, top*cpu2p, right*cpu2p, bottom*cpu2p
		#endif
		
		top = 10
		CheckBox checkLoadAB, pos={10,top}, title = "Load absorbance spectrum", Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox checkLoadTR, pos={10,top}, title = "Load transmittance spectrum", Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox checkLoadREF, pos={10,top}, title = "Load background spectrum", Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox checkLoadSC, pos={10,top}, title = "Load single channel spectrum", Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox checkLoadIFM, pos={10,top}, title = "Load interferograms", Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox checkLoadIMG, pos={10,top}, title = "Load photographic images", Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox checkSettings, pos={10,top}, title = "More options", mode=2, value=0, Proc=opusloader#CheckBoxProc
		top += 30
		Button btnCancel, title="Cancel",pos={100,top},size={55,20}, Proc=OpusLoader#buttonProc
		Button btnContinue, title="Continue",pos={165,top},size={70,20}, Proc=OpusLoader#buttonProc
		
		// hideable controls for setting more advanced preferences
		
		top = 150
		Button xbtnDefaults, title="Set to Default Values",pos={10,top},size={140,18}, Proc=OpusLoader#buttonProc, disable=1
		
		top += 30
		SetVariable xsvABprefix, pos={10,top}, size = {160,20}, bodyWidth=30, title = "Absorbance wave prefix", disable = 1, Proc=OpusLoader#SetVariableProc
		SetVariable xsvABsuffix, pos={175,top}, size = {60,20}, bodyWidth=30, title = "suffix", disable = 1, Proc=OpusLoader#SetVariableProc
		top += 20
		SetVariable xsvTRprefix, pos={10,top}, size = {160,20}, bodyWidth=30, title = "Transmittance prefix", disable = 1, Proc=OpusLoader#SetVariableProc
		SetVariable xsvTRsuffix, pos={175,top}, size = {60,20}, bodyWidth=30, title = "suffix", disable = 1, Proc=OpusLoader#SetVariableProc
		top += 20
		SetVariable xsvSCprefix, pos={10,top}, size = {160,20}, bodyWidth=30, title = "Single Channel wave prefix", disable = 1, Proc=OpusLoader#SetVariableProc
		SetVariable xsvSCsuffix, pos={175,top}, size = {60,20}, bodyWidth=30, title = "suffix", disable = 1, Proc=OpusLoader#SetVariableProc
		top += 20
		SetVariable xsvREFprefix, pos={10,top}, size = {160,20}, bodyWidth=30, title = "Background wave prefix", disable = 1, Proc=OpusLoader#SetVariableProc
		SetVariable xsvREFsuffix, pos={175,top}, size = {60,20}, bodyWidth=30, title = "suffix", disable = 1, Proc=OpusLoader#SetVariableProc
		top += 20
		SetVariable xsvIFSprefix, pos={10,top}, size = {160,20}, bodyWidth=30, title = "Interferogram [SC] prefix", disable = 1, Proc=OpusLoader#SetVariableProc
		SetVariable xsvIFSsuffix, pos={175,top}, size = {60,20}, bodyWidth=30, title = "suffix", disable = 1, Proc=OpusLoader#SetVariableProc
		top += 20
		SetVariable xsvIFRprefix, pos={10,top}, size = {160,20}, bodyWidth=30, title = "Interferogram [REF] prefix", disable = 1, Proc=OpusLoader#SetVariableProc
		SetVariable xsvIFRsuffix, pos={175,top}, size = {60,20}, bodyWidth=30, title = "suffix", disable = 1, Proc=OpusLoader#SetVariableProc
		top += 20
		SetVariable xsvIMGprefix, pos={10,top}, size = {160,20}, bodyWidth=30, title = "Image prefix", disable = 1, Proc=OpusLoader#SetVariableProc
		SetVariable xsvIMGsuffix, pos={175,top}, size = {60,20}, bodyWidth=30, title = "suffix", disable = 1, Proc=OpusLoader#SetVariableProc
		top += 20
		CheckBox xchkFolders, pos={10,top}, title = "Save SC, BG and IFM waves in subfolders", disable = 1, Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox xchkExtension, pos={10,top}, title = "Use file extension in wave name", disable = 1, Proc=opusloader#CheckBoxProc
		top += 20
		TitleBox xTitleFPA, pos={10,top+2}, frame=0, title = "Save 2D data as:", disable = 1
		CheckBox xchkMatrix, pos={100,top}, mode=1, title = "matrix", disable = 1, Proc=opusloader#CheckBoxProc
		CheckBox xchkSplit, pos={160,top}, mode=1, title = "spectra", disable = 1, Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox xchkDisplayStack, pos={10,top}, title = "Display hyperspectral stack", disable = 1, Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox xchkDisplayImage, pos={10,top}, title = "Display photographic images", disable = 1, Proc=opusloader#CheckBoxProc
		top += 20
		PopupMenu xpopOverwrite, pos={10,top}, value ="rename;overwrite;ask;", title = "If waves exist", disable = 1, Proc=OpusLoader#PopupProc
		top += 20
		TitleBox xTitle, pos={10,top}, frame=0, anchor=LC, title = "Print to History Settings:", disable = 1
		top += 20
		CheckBox xchkVerbose1, pos={10,top}, title = "Input file info", disable = 1, Proc=opusloader#CheckBoxProc
		CheckBox xchkVerbose2, pos={130,top}, title = "Output waves", disable = 1, Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox xchkVerbose4, pos={10,top}, title = "OPUS parameters", disable = 1, Proc=opusloader#CheckBoxProc
		CheckBox xchkVerbose8, pos={130,top}, title = "OPUS file info", disable = 1, Proc=opusloader#CheckBoxProc
		top += 20
		CheckBox xchkVerbose16, pos={10,top}, title = "OPUS file directory, full", disable = 1, Proc=opusloader#CheckBoxProc
		
		prefs.loadoptions = prefs.loadoptions & ~2^15 // clear bit 15
		SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
		string strPrefs
		StructPut/S prefs strPrefs
		SetWindow OpusLoaderPanel userdata(prefs) = strPrefs
		SetControlValues() // sets control values based on prefs structure stored as panel userdata
		SetWindow OpusLoaderPanel userdata(expand) = "0"
				
		PauseForUser OpusLoaderPanel // updated prefs are saved when panel is killed with 'continue' button
		
		LoadPrefs(prefs) // reload saved prefs
		if ( ! (prefs.loadoptions & 2^15)) // cancel
			return 0
		endif
	endif
	
	// present dialog to select files
	int refnum
	string fileFilters = "All Files:.*;Data Files (*.txt,*.dat,*.csv):.txt,.dat,.csv;Weird files (*.0):.0;"
	Open/R/D/MULT=1/F=fileFilters/M="Looking for an OPUS file..." refnum
	if (!strlen(s_filename))
		return 0
	endif
		
	string fileToLoad = ""
	int numfiles = ItemsInList(S_filename, "\r")
	int i
	for (i=0;i<numfiles;i++)
		fileToLoad = StringFromList(i, S_filename, "\r")
		LoadOpusFile(fileToLoad, prefs)
	endfor

	return 0
end

// point and rect structures must have same units
function PointInRect(STRUCT point &pnt, STRUCT rect &r)
	return ( pnt.h>r.left && pnt.h<r.right && pnt.v>r.top && pnt.v<r.bottom )
end

function GetScreenOrigin(STRUCT point &pnt)
	
	// set default value
	pnt.h = 0
	pnt.v = 0
	
	GetMouse
	
	#ifdef debug
	Print v_left, v_top
	#endif

	STRUCT point mousePoint
	mousePoint.h = v_left
	mousePoint.v = v_top
	
	STRUCT rect screenRect
	
	string strInfo = IgorInfo(0)
	string str = ""
	int numScreens = NumberByKey("NSCREENS", strInfo)
	int i, left, top, right, bottom
	
	for (i=1;i<=numScreens;i++)
		str = StringByKey("SCREEN"+num2str(i), strInfo)
		sscanf str, "DEPTH=%*d,RECT=%d,%d,%d,%d", left, top, right, bottom
		if (V_flag != 4)
			continue
		endif
		screenRect.left   = left
		screenRect.top    = top
		screenRect.right  = right
		screenRect.bottom = bottom
		if ( PointInRect(mousePoint, screenRect) )
			pnt.h = screenRect.left
			pnt.v = screenRect.top
			return 1
		endif
	endfor
	return 0
end

function ButtonProc(STRUCT WMButtonAction &s)
	if (s.eventCode != 2)
		return 0
	endif
	
	strswitch (s.ctrlName)
		case "btnCancel" :
			KillWindow/Z OpusLoaderPanel
			break
		case "btnContinue" :
			// collect prefs
			STRUCT PackagePrefs prefs
			StructGet/S prefs GetUserData(s.win, "", "prefs")
			prefs.loadoptions = prefs.loadoptions | 2^15 // set continue bit
			// save prefs from window before killing panel
			SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
			KillWindow/Z OpusLoaderPanel
			break
		case "xbtnDefaults" :
			StructGet/S prefs GetUserData(s.win, "", "prefs")
			PrefsSetDefaults(prefs)
			string strPrefs
			StructPut/S prefs strPrefs
			SetWindow $s.win userdata(prefs) = strPrefs
			SetControlValues()
			break
	endswitch
	return 0
end

// sets control values to match preferences saved in window userdata
function SetControlValues()
	
	STRUCT PackagePrefs prefs
	StructGet/S prefs GetUserData("OpusLoaderPanel", "", "prefs")
	
	CheckBox checkLoadAB,  win=OpusLoaderPanel, value = prefs.loadoptions & 1
	CheckBox checkLoadREF, win=OpusLoaderPanel, value = prefs.loadoptions & 2
	CheckBox checkLoadSC,  win=OpusLoaderPanel, value = prefs.loadoptions & 4
	CheckBox checkLoadIFM, win=OpusLoaderPanel, value = prefs.loadoptions & 8
	CheckBox checkLoadTR,  win=OpusLoaderPanel, value = prefs.loadoptions & 16
	CheckBox checkLoadIMG, win=OpusLoaderPanel, value = prefs.loadoptions & 32
	
	SetVariable xsvABprefix,  win=OpusLoaderPanel, value = _STR:prefs.prefix.AB
	SetVariable xsvABsuffix,  win=OpusLoaderPanel, value = _STR:prefs.suffix.AB
	SetVariable xsvSCprefix,  win=OpusLoaderPanel, value = _STR:prefs.prefix.SC
	SetVariable xsvSCsuffix,  win=OpusLoaderPanel, value = _STR:prefs.suffix.SC
	SetVariable xsvREFprefix, win=OpusLoaderPanel, value = _STR:prefs.prefix.REF
	SetVariable xsvREFsuffix, win=OpusLoaderPanel, value = _STR:prefs.suffix.REF
	SetVariable xsvIFSprefix, win=OpusLoaderPanel, value = _STR:prefs.prefix.IFS
	SetVariable xsvIFSsuffix, win=OpusLoaderPanel, value = _STR:prefs.suffix.IFS
	SetVariable xsvIFRprefix, win=OpusLoaderPanel, value = _STR:prefs.prefix.IFR
	SetVariable xsvIFRsuffix, win=OpusLoaderPanel, value = _STR:prefs.suffix.IFR
	SetVariable xsvTRprefix,  win=OpusLoaderPanel, value = _STR:prefs.prefix.TR
	SetVariable xsvTRsuffix,  win=OpusLoaderPanel, value = _STR:prefs.suffix.TR
	SetVariable xsvIMGprefix, win=OpusLoaderPanel, value = _STR:prefs.prefix.IMG
	SetVariable xsvIMGsuffix, win=OpusLoaderPanel, value = _STR:prefs.suffix.IMG
	
	CheckBox xchkFolders,      win=OpusLoaderPanel, value = prefs.options & 1
	CheckBox xchkExtension,    win=OpusLoaderPanel, value = prefs.options & 2
	CheckBox xchkMatrix,       win=OpusLoaderPanel, value = !(prefs.options & 4)
	CheckBox xchkSplit,        win=OpusLoaderPanel, value = prefs.options & 4
	CheckBox xchkDisplayStack, win=OpusLoaderPanel, value = prefs.options & 8
	CheckBox xchkDisplayImage, win=OpusLoaderPanel, value = prefs.options & 16

	CheckBox xchkVerbose1,  win=OpusLoaderPanel, value = prefs.verbose & 1
	CheckBox xchkVerbose2,  win=OpusLoaderPanel, value = prefs.verbose & 2
	CheckBox xchkVerbose4,  win=OpusLoaderPanel, value = prefs.verbose & 4
	CheckBox xchkVerbose8,  win=OpusLoaderPanel, value = prefs.verbose & 8
	CheckBox xchkVerbose16, win=OpusLoaderPanel, value = prefs.verbose & 16
		
	PopupMenu xpopOverwrite, win=OpusLoaderPanel, mode=(prefs.overwrite+1)
	ControlUpdate/A/W=OpusLoaderPanel
end

function CheckBoxProc(STRUCT WMCheckBoxAction &s) : CheckboxControl
	
	if (s.eventCode != 2)
		return 0
	endif
	
	STRUCT PackagePrefs prefs
	StructGet/S prefs GetUserData(s.win, "", "prefs")

	strswitch (s.ctrlName)
		case "xchkMatrix" :
			prefs.options = prefs.options & ~4
			CheckBox xchkSplit, win=$s.win, value=0
			break
		case "xchkSplit" :
			prefs.options = prefs.options | 4
			CheckBox xchkMatrix, win=$s.win, value=0
			break
		case "checkLoadAB" :
			prefs.loadoptions = (prefs.loadoptions & ~1) | (1*s.checked)
			break
		case "checkLoadREF" :
			prefs.loadoptions = (prefs.loadoptions & ~2) | (2*s.checked)
			break
		case "checkLoadSC" :
			prefs.loadoptions = (prefs.loadoptions & ~4) | (4*s.checked)
			break
		case "checkLoadIFM" :
			prefs.loadoptions = (prefs.loadoptions & ~8) | (8*s.checked)
			break
		case "checkLoadTR" :
			prefs.loadoptions = (prefs.loadoptions & ~16) | (16*s.checked)
			break
		case "checkLoadIMG" :
			prefs.loadoptions = (prefs.loadoptions & ~32) | (32*s.checked)
			break
		case "checkSettings" : // disclosure box to extend panel
			GetWindow $s.win wsizeRM // points
			variable height = V_bottom - V_top
			variable factor = height / (s.checked ? 190 : 190+360)

			V_bottom = s.checked ? V_bottom + 360*factor : V_bottom - 360*factor
			MoveWindow/W=$s.win V_left, V_top, V_right, V_bottom // points
			
			// control panel units
			Button btnContinue, win=$s.win, pos={165, 160 + 360*s.checked}
			Button btnCancel, win=$s.win, pos={100, 160 + 360*s.checked}
			
			// set hidden controls visibility
			ModifyControlList ControlNameList(s.win, ";", "x*") win=$s.win, disable=!s.checked
			break
		case "xchkFolders" :
			prefs.options = (prefs.options & ~1) | (1*s.checked)
			break
		case "xchkExtension" :
			prefs.options = (prefs.options & ~2) | (2*s.checked)
			break
			
		case "xchkDisplayStack" :
			prefs.options = (prefs.options & ~8) | (8*s.checked)
			break
		case "xchkDisplayImage" :
			prefs.options = (prefs.options & ~16) | (16*s.checked)
			break
			
		case "xchkVerbose1" :
			prefs.verbose = (prefs.verbose & ~1) | (1*s.checked)
			break
		case "xchkVerbose2" :
			prefs.verbose = (prefs.verbose & ~2) | (2*s.checked)
			break
		case "xchkVerbose4" :
			prefs.verbose = (prefs.verbose & ~4) | (4*s.checked)
			break
		case "xchkVerbose8" :
			prefs.verbose = (prefs.verbose & ~8) | (8*s.checked)
			break
		case "xchkVerbose16" :
			prefs.verbose = (prefs.verbose & ~16) | (16*s.checked)
			break
		
	endswitch
	
	string strPrefs
	StructPut/S prefs strPrefs
	SetWindow $s.win userdata(prefs) = strPrefs
	
	return 0
end

function PopupProc(STRUCT WMPopupAction &s)
	
	if (s.eventCode != 2)
		return 0
	endif
	
	STRUCT PackagePrefs prefs
	StructGet/S prefs GetUserData(s.win, "", "prefs")
	
	prefs.overwrite = s.popNum - 1
	
	string strPrefs
	StructPut/S prefs strPrefs
	SetWindow $s.win userdata(prefs) = strPrefs
	
	return 0
end


function SetVariableProc(STRUCT WMSetVariableAction &s)

	if (s.eventcode != 8) // end edit
		return 0
	endif

	s.sval = (s.sval)[0,5]
	SetVariable $s.ctrlName win=$s.win, value=_STR:s.sval
	
	STRUCT PackagePrefs prefs
	StructGet/S prefs GetUserData(s.win, "", "prefs")
	
	strswitch (s.ctrlName)
		case "xsvABprefix" :
			prefs.prefix.AB = s.sval
			break
		case "xsvABsuffix" :
			prefs.suffix.AB = s.sval
			break
		case "xsvSCprefix" :
			prefs.prefix.SC = s.sval
			break
		case "xsvSCsuffix" :
			prefs.suffix.SC = s.sval
			break
		case "xsvREFprefix" :
			prefs.prefix.REF = s.sval
			break
		case "xsvREFsuffix" :
			prefs.suffix.REF = s.sval
			break
		case "xsvIFSprefix" :
			prefs.prefix.IFS = s.sval
			break
		case "xsvIFSsuffix" :
			prefs.suffix.IFS = s.sval
			break
		case "xsvIMGprefix" :
			prefs.prefix.IMG = s.sval
			break
		case "xsvIMGsuffix" :
			prefs.suffix.IMG = s.sval
			break
	endswitch
	
	string str
	StructPut/S prefs str
	SetWindow $s.win userdata(prefs) = str
	
	return 0
end

function LoadOpusFile(string strFile, STRUCT PackagePrefs &prefs)
		
	int block, refnum
	int absLoaded = 0 // set this after loading first absorbance block
	// if we find another, ok to overwrite, whatever the value of overwrite
	
	// open the data file
	Open/Z/R refnum as strFile
	if (V_flag != 0)
		return V_flag
	endif
	
	// extract info from header block
	int MagicNumber, version, Pointer, MaxSize, DirSize
	FSetPos refNum, 0
	FBinRead/B=2/F=3 refnum, MagicNumber
	if (MagicNumber != 0x0A0AFEFE)
		printf "%s doesn't appear to be a valid Bruker file\r" ParseFilePath(0, strFile, ":", 1, 0)
		return 0
	endif
	FBinRead/B=3/F=5 refnum, version // always the same value, 920622
	FBinRead/U/B=3/F=3 refnum, Pointer
	FBinRead/U/B=3/F=3 refnum, MaxSize
	FBinRead/U/B=3/F=3 refnum, DirSize
	
	FSetPos refNum, Pointer // first directory block
	int BlockType, BlockLength, BlockPointer
	
	string newName = "" // use newName string to build new wave names
	string baseName = ParseFilePath(3, S_filename, ":", 0, 0)
	if (prefs.options & 2)
		baseName =  ParseFilePath(0, S_filename, ":", 1, 0)
	endif
	// read parameter blocks into strParameters as nested keylists
	string strParameters = "", DirectoryInfo = "", ExtendedDirInfo = ""
	string strItem = "", cmd = "", key = "", strExtnd = ""
	
	int CPLX, PARM, STYP, DATA, DERIV, EXTND
	int loaded
	int numImages = 0
	
	PrintToHistory("\r  ***  Loading " + S_filename + "  ***", prefs.verbose & 1)
	
	for (block=0; block<DirSize; block+=1) // read directory info
		
		loaded = 0
		DirectoryInfo = ""
		ExtendedDirInfo = ""
		strExtnd = ""
		
		FBinRead/U/B=3/F=3 refnum, BlockType
		FBinRead/U/B=3/F=3 refnum, BlockLength
		FBinRead/U/B=3/F=3 refnum, BlockPointer
		
		DirectoryInfo = ReplaceNumberByKey("Block", DirectoryInfo, block, "=", ";")
		DirectoryInfo = ReplaceNumberByKey("BlockType", DirectoryInfo, BlockType, "=", ";")
		DirectoryInfo = ReplaceNumberByKey("BlockLength", DirectoryInfo, BlockLength, "=", ";")
		DirectoryInfo = ReplaceNumberByKey("BlockPointer", DirectoryInfo, BlockPointer, "=", ";")
			
		// examine bits 0 and 1
		CPLX = BlockType & 3
		if (CPLX)
			DirectoryInfo = ReplaceNumberByKey("CPLX", DirectoryInfo, CPLX, "=", ";")
			ExtendedDirInfo = AddListItem(CPLXTypeString(CPLX), ExtendedDirInfo, ";" , 0)
		endif
		
		// examine bits 2 and 3
		STYP = (BlockType >> 2) & 3
		if (STYP)
			DirectoryInfo = ReplaceNumberByKey("STYP", DirectoryInfo, STYP, "=", ";")
			ExtendedDirInfo = AddListItem(SpectrumTypeString(STYP), ExtendedDirInfo, ";" , 0)
		endif
		
		// examine bits 4 - 9
		PARM = (BlockType >> 4) & 0x3F
		if (PARM)
			DirectoryInfo = ReplaceNumberByKey("PARM", DirectoryInfo, PARM, "=", ";")
			ExtendedDirInfo = AddListItem(ParameterTypeString(PARM), ExtendedDirInfo, ";" , 0)
		endif
		
		// examine bits 10 - 16
		DATA = (BlockType >> 10) & 0x7F
		if (DATA)
			DirectoryInfo = ReplaceNumberByKey("DATA", DirectoryInfo, DATA, "=", ";")
			if (PARM == 0 && DATA != 13)
				ExtendedDirInfo = AddListItem("Data", ExtendedDirInfo, ";" , 0)
			endif
			ExtendedDirInfo = AddListItem(DataTypeString(data), ExtendedDirInfo, ";" , Inf)
		endif
			
		// examine bits 17 - 18
		DERIV = (BlockType >> 17) & 3
		if (DERIV)
			DirectoryInfo = ReplaceNumberByKey("DERIV", DirectoryInfo, DERIV, "=", ";")
		endif
	
		// examine bits 19 - 25
		EXTND = (BlockType >> 19) & 0x7F
		if (EXTND)
			DirectoryInfo = ReplaceNumberByKey("EXTND", DirectoryInfo, EXTND, "=", ";")
			ExtendedDirInfo = AddListItem(ExtendedTypeString(EXTND), ExtendedDirInfo, ";" , Inf)
		endif
		
		// prepend block number to the human readable list
		ExtendedDirInfo = AddListItem("Block " + num2str(block), ExtendedDirInfo, ";" , 0)
		
		PrintToHistory("\r" + ExtendedDirInfo, prefs.verbose & 8)
		PrintToHistory(DirectoryInfo, prefs.verbose & 16)
				
		// check for (and potentially load) a data block
		if (PARM==0 && CPLX==3 && (EXTND==0 || EXTND==10) && DERIV==0) // potentially useful data
			if (STYP==3 && DATA==4) // absorbance data
				if (prefs.loadOptions & 1)
					newName = prefs.prefix.AB + BaseName + prefs.suffix.AB
					// if we find more than one absorbance block, keep overwriting absorbance wave
					if (absLoaded)
						PrintToHistory("* Found multiple AB data blocks, reloading AB from block " + num2str(block), prefs.verbose & 24)
					endif
					
					if (EXTND == 0) // single spectrum
						wave/Z w_abs = LoadDataBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite)
						absLoaded = WaveExists(w_abs)
					elseif (prefs.options & 4)
						wave/Z/wave wrefs_abs = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 1)
						absLoaded = WaveExists(wrefs_abs) && numpnts(wrefs_abs)
					else
						wave/Z m_abs = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 0)
						absLoaded = WaveExists(m_abs)
					endif
					loaded = absLoaded
				endif
			elseif (STYP==3 && DATA==5) // transmittance data
				if (prefs.loadOptions & 16)
					newName = prefs.prefix.TR + BaseName + prefs.suffix.TR
					
					if (EXTND==0) // single spectrum
						wave/Z w_tr = LoadDataBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite)
						loaded = WaveExists(w_tr)
					elseif (prefs.options & 4)
						wave/Z/wave wrefs_tr = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 1)
						loaded = WaveExists(wrefs_tr) && numpnts(wrefs_tr)
					else
						wave/Z m_tr = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 0)
						loaded = WaveExists(m_tr)
					endif
				endif
			elseif (STYP==1 && DATA==1)
				if (prefs.loadOptions & 4)
					newName = prefs.prefix.SC + BaseName + prefs.suffix.SC
					newName = SelectString(prefs.options & 1, newName, ":SingleChannelSpectra:" + NewName)
					if (EXTND==0) // single spectrum
						wave/Z w_sc = LoadDataBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite)
						loaded = WaveExists(w_sc)
					elseif (prefs.options & 4)
						wave/Z/wave wrefs_sc = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 1)
						loaded = WaveExists(wrefs_sc) && numpnts(wrefs_sc)
					else
						wave/Z m_sc = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, prefs.overwrite, 0)
						loaded = WaveExists(m_sc)
					endif
				endif
			elseif (STYP==2 && DATA==1)
				if (prefs.loadOptions & 2)
					newName = prefs.prefix.REF + BaseName + prefs.suffix.REF
					newName = SelectString(prefs.options & 1, newName, ":BackgroundSpectra:" + NewName)
					
					if (EXTND == 0) // single spectrum
						wave/Z w_ref = LoadDataBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite)
						loaded = WaveExists(w_ref)
					elseif (prefs.options & 4)
						wave/Z/wave wrefs_ref = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 1)
						loaded = WaveExists(wrefs_ref) && numpnts(wrefs_ref)
					else
						wave/Z m_ref = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, prefs.overwrite, 0)
						loaded = WaveExists(m_ref)
					endif
				endif
			elseif (STYP==1 && DATA==2)
				if (prefs.loadOptions & 8)
					newName = prefs.prefix.IFS + BaseName + prefs.suffix.IFS
					newName = SelectString(prefs.options & 1, newName, ":Interferograms:" + NewName)
					
					if (EXTND == 0) // single spectrum
						wave/Z w_ifs = LoadDataBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite)
						loaded = WaveExists(w_ifs)
					elseif (prefs.options & 4)
						wave/Z/wave wrefs_ifs = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 1)
						loaded = WaveExists(wrefs_ifs) && numpnts(wrefs_ifs)
					else
						wave/Z m_ifs = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, prefs.overwrite, 0)
						loaded = WaveExists(m_ifs)
					endif
				endif
			elseif (STYP==2 && DATA==2)
				if (prefs.loadOptions & 8)
					newName = prefs.prefix.IFR + BaseName + prefs.suffix.IFR
					newName = SelectString(prefs.options & 1, newName, ":Interferograms:" + NewName)
					
					if (EXTND == 0) // single spectrum
						wave/Z w_ifr = LoadDataBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite)
						loaded = WaveExists(w_ifr)
					elseif (prefs.options & 4)
						wave/Z/wave wrefs_ifr = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, absLoaded ? 1 : prefs.overwrite, 1)
						loaded = WaveExists(wrefs_ifr) && numpnts(wrefs_ifr)
					else
						wave/Z m_ifr = LoadMatrixBlock(BlockPointer, BlockLength, refnum, newName, prefs.overwrite, 0)
						loaded = WaveExists(m_ifr)
					endif
				endif
			endif
		endif
				
		// check for and load parameter block
		switch (PARM) // maybe need to exclude DERIV>0 and EXTND>0 blocks???
			case 1: // data parameters
				if (CPLX == 3) // potentially useful data parameters
					key = DataParameterString(STYP, DATA)
					if (!strlen(key))
						break
					endif
					strItem = LoadParameterBlock(BlockPointer, BlockLength, refnum)
					strParameters = ReplaceStringByKey(key, strParameters, strItem, ":", "\r")
					loaded = 1
				endif
				break
			case 2: // instrument parameters
			case 3: // acquisition parameters
			case 4: // FT parameters
			case 6: // processing parameters = optic parameters
				strItem = LoadParameterBlock(BlockPointer, BlockLength, refnum)
				key =  ParameterTypeString(PARM) + SelectString(STYP == 2, " [SC]", " [REF]")
				if (strlen(strItem))
					strParameters = ReplaceStringByKey(key, strParameters, strItem, ":", "\r")
					loaded = 1
				endif
				break
			case 10: // sample origin parameters
				strItem = LoadParameterBlock(BlockPointer, BlockLength, refnum)
				if (strlen(strItem))
					strParameters = ReplaceStringByKey("Sample Parameters", strParameters, strItem, ":", "\r")
					loaded = 1
				endif
				break
		endswitch
		
		string strImage = ""
		switch (EXTND)
			case 13 :
				strExtnd = LoadTextBlock(BlockPointer, BlockLength, refnum)
				break
			case 11 :
				strExtnd = LoadImageParameterBlock(BlockPointer, BlockLength, refnum)
				break
			case 20:
				strExtnd = LoadImageParameterBlock(BlockPointer, BlockLength, refnum)
				wave/Z/U/B wImage = LoadImageBlock(BlockPointer, BlockLength, refnum)
				break
		endswitch
		
		if (strlen(strExtnd))
			strParameters = ReplaceStringByKey(ExtendedTypeString(EXTND), strParameters, strExtnd, ":", "\r")
			loaded = 1
		endif
		
		if ((prefs.loadOptions&32) && WaveExists(wImage))
			// save image
			newName = CleanupName(baseName, 0)
			if (NumImages)
				newName += "_" + num2str(NumImages)
			endif
			newName = prefs.prefix.IMG + newName + prefs.suffix.IMG
			newName = SelectString(prefs.options & 1, newName, ":Images:" + NewName)
						
			wave/Z img = SaveImage(wImage, newName, prefs.overwrite)
			wave/Z wImage = $""
			
			if (WaveExists(img))
				numImages += 1
				if (strlen(strExtnd))
					note img, ExtendedTypeString(EXTND) + ":" + strExtnd
					variable x0, y0, delx, dely
					// this may be flakey, should probably find the two keypairs that define title and value
					x0 = NumberByKey("V01", strExtnd, "=")
					y0 = NumberByKey("V02", strExtnd, "=")
					delx = NumberByKey("V06", strExtnd, "=")
					dely = NumberByKey("V07", strExtnd, "=")
					
					if ((numtype(x0) + numtype(y0) + numtype(delx) + numtype(dely)) == 0)
						SetScale/P x, x0, delx, img
						SetScale/P y, y0, dely, img
					endif
					
				endif
				
				string str
				sprintf str, "%d x %d image saved as %s\r", DimSize(img, 0), DimSize(img, 1), GetWavesDataFolder(img, 4)
				PrintToHistory(str, prefs.verbose & 2)
								
				if (prefs.options & 16)
					NewImage/K=1 img
				endif
			endif
			
		endif
				
		PrintToHistory("Loaded block " + num2str(block), loaded && (prefs.verbose & 16))
				
 	endfor // bottom of loop through directory
	
	Close refNum
	
	strParameters = SortList(strParameters, "\r")
	string strNote = ""
	
	// clean up the extracted data blocks
	variable FXV, LXV
	int havelimits = 0
	int i, numSpectra
	
	if (prefs.loadOptions & 1) // trying to load an absorbance spectrum
		strItem = StringByKey("Data Parameters [AB]", strParameters, ":", "\r")
		FXV = NumberByKey("FXV", strItem, "=")
		LXV = NumberByKey("LXV", strItem, "=")
		havelimits = numtype(FXV)==0 && numtype(LXV)==0
		if (havelimits && WaveExists(w_abs))
			SetScale/I x FXV, LXV, w_abs
			note w_abs, RemoveEnding(strParameters, "\r")
			sprintf cmd, "Created absorbance spectrum %s from file %s\r" NameOfWave(w_abs), S_FileName
		elseif (havelimits && WaveExists(m_abs))
			SetScale/I z FXV, LXV, m_abs
			note m_abs, RemoveEnding(strParameters, "\r")
			sprintf cmd, "Created absorbance matrix %s from file %s\r" NameOfWave(m_abs), S_FileName
		elseif (havelimits && WaveExists(wrefs_abs) && numpnts(wrefs_abs))
			numSpectra = numpnts(wrefs_abs)
			for (i=0;i<numSpectra;i++)
				SetScale/I x FXV, LXV, wrefs_abs[i]
				note wrefs_abs[i], RemoveEnding(strParameters, "\r")
			endfor
			sprintf cmd, "Created %d absorbance spectra from file %s\r" numSpectra, S_FileName
		else
			KillWaves/Z w_abs, m_abs, wrefs_abs
			sprintf cmd, "Absorbance spectrum not loaded from %s\r" S_FileName
		endif
		PrintToHistory(cmd, prefs.verbose & 2)
		
		string strWavePaths = ""
		if ((prefs.loadOptions & 2) && WaveExists(w_ref))
			strWavePaths = ReplaceStringByKey("REF", strWavePaths, GetWavesDataFolder(w_ref, 2), "=", ";")
		endif
		
		if ((prefs.loadOptions & 4) && WaveExists(w_sc))
			strWavePaths = ReplaceStringByKey("SC", strWavePaths, GetWavesDataFolder(w_sc, 2), "=", ";")
		endif
		
		if ((prefs.loadOptions & 8))
			if (WaveExists(w_ifs))
				strWavePaths = ReplaceStringByKey("IFS", strWavePaths, GetWavesDataFolder(w_ifs, 2), "=", ";")
			endif
			if (WaveExists(w_ifr))
				strWavePaths = ReplaceStringByKey("IFR", strWavePaths, GetWavesDataFolder(w_ifr, 2), "=", ";")
			endif
		endif
		
		if (strlen(strWavePaths) && WaveExists(w_abs))
			strWavePaths = ReplaceStringByKey("Associated Waves", "", strWavePaths, ":", "\r")
			note w_abs, strWavePaths
		endif
		
	endif
		
	if (prefs.loadOptions & 16) // trying to load a transmittance spectrum
		strItem = StringByKey("Data Parameters [TR]", strParameters, ":", "\r")
		FXV = NumberByKey("FXV", strItem, "=")
		LXV = NumberByKey("LXV", strItem, "=")
		havelimits = numtype(FXV)==0 && numtype(LXV)==0
		if (havelimits && WaveExists(w_tr))
			SetScale/I x FXV, LXV, w_tr
			note w_tr, RemoveEnding(strParameters, "\r")
			sprintf cmd, "Created transmittance spectrum %s from file %s\r" NameOfWave(w_tr), S_FileName
		elseif (havelimits && WaveExists(m_tr))
			SetScale/I z FXV, LXV, m_tr
			note m_tr, RemoveEnding(strParameters, "\r")
			sprintf cmd, "Created transmittance matrix %s from file %s\r" NameOfWave(m_tr), S_FileName
		elseif (havelimits && WaveExists(wrefs_tr) && numpnts(wrefs_tr))
			numSpectra = numpnts(wrefs_tr)
			for (i=0;i<numSpectra;i++)
				SetScale/I x FXV, LXV, wrefs_tr[i]
				note wrefs_tr[i], RemoveEnding(strParameters, "\r")
			endfor
			sprintf cmd, "Created %d transmittance spectra from file %s\r" numSpectra, S_FileName
		else
			KillWaves/Z w_tr, m_tr, wrefs_tr
			sprintf cmd, "Transmittance spectrum not loaded from %s\r" S_FileName
		endif
		PrintToHistory(cmd, prefs.verbose & 2)
		
		strWavePaths = ""
		if ((prefs.loadOptions & 2) && WaveExists(w_ref))
			strWavePaths = ReplaceStringByKey("REF", strWavePaths, GetWavesDataFolder(w_ref, 2), "=", ";")
		endif
		
		if ((prefs.loadOptions & 4) && WaveExists(w_sc))
			strWavePaths = ReplaceStringByKey("SC", strWavePaths, GetWavesDataFolder(w_sc, 2), "=", ";")
		endif
		
		if ((prefs.loadOptions & 8))
			if (WaveExists(w_ifs))
				strWavePaths = ReplaceStringByKey("IFS", strWavePaths, GetWavesDataFolder(w_ifs, 2), "=", ";")
			endif
			if (WaveExists(w_ifr))
				strWavePaths = ReplaceStringByKey("IFR", strWavePaths, GetWavesDataFolder(w_ifr, 2), "=", ";")
			endif
		endif
		
		if (strlen(strWavePaths) && WaveExists(w_tr))
			strWavePaths = ReplaceStringByKey("Associated Waves", "", strWavePaths, ":", "\r")
			note w_tr, strWavePaths
		endif
		
	endif
			
	if (prefs.loadOptions & 2) // trying to load a reference spectrum
		strItem = StringByKey("Data Parameters [REF]", strParameters, ":", "\r")
		FXV = NumberByKey("FXV", strItem, "=")
		LXV = NumberByKey("LXV", strItem, "=")
				
		havelimits = numtype(FXV)==0 && numtype(LXV)==0
		strNote  = KeypairByKey("Acquisition Parameters [REF]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("FT Parameters [REF]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Optic Parameters [REF]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Data Parameters [REF]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Instrument Parameters [REF]", strParameters, ":", "\r")
		
		if (havelimits && WaveExists(w_ref))
			SetScale/I x FXV, LXV, w_ref
			note w_ref, strNote
			sprintf cmd, "Created reference spectrum %s from file %s\r" NameOfWave(w_ref), S_FileName
		elseif (havelimits && WaveExists(m_ref))
			SetScale/I z FXV, LXV, m_ref
			note m_ref, strNote
			sprintf cmd, "Created reference spectrum matrix %s from file %s\r" NameOfWave(m_ref), S_FileName
		elseif (havelimits && WaveExists(wrefs_ref) && numpnts(wrefs_ref))
			numSpectra = numpnts(wrefs_ref)
			for (i=0;i<numSpectra;i++)
				SetScale/I x FXV, LXV, wrefs_ref[i]
				note wrefs_ref[i], strNote
			endfor
			sprintf cmd, "Created %d reference spectra from file %s\r" numSpectra, S_FileName
		else
			KillWaves/Z w_ref, m_ref, wrefs_ref
			sprintf cmd, "Reference spectrum not loaded from %s\r" S_FileName
		endif
		PrintToHistory(cmd, prefs.verbose & 2)
	endif
	
	if (prefs.loadOptions & 4) // trying to load a single channel spectrum
		strItem = StringByKey("Data Parameters [SC]", strParameters, ":", "\r")
		FXV = NumberByKey("FXV", strItem, "=")
		LXV = NumberByKey("LXV", strItem, "=")
		
		havelimits = numtype(FXV)==0 && numtype(LXV)==0
		strNote  = KeypairByKey("Sample Parameters", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Acquisition Parameters [SC]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("FT Parameters [SC]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Optic Parameters [SC]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Data Parameters [SC]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Instrument Parameters [SC]", strParameters, ":", "\r")
		
		if (havelimits && WaveExists(w_sc))
			SetScale/I x FXV, LXV, w_sc
			note w_sc, strNote
			sprintf cmd, "Created single channel spectrum %s from file %s\r" NameOfWave(w_sc), S_FileName
		elseif (havelimits && WaveExists(m_sc))
			SetScale/I z FXV, LXV, m_sc
			note m_sc, strNote
			sprintf cmd, "Created single channel matrix %s from file %s\r" NameOfWave(m_sc), S_FileName
		elseif (havelimits && WaveExists(wrefs_sc) && numpnts(wrefs_sc))
			numSpectra = numpnts(wrefs_sc)
			for (i=0;i<numSpectra;i++)
				SetScale/I x FXV, LXV, wrefs_sc[i]
				note wrefs_sc[i], strNote
			endfor
			sprintf cmd, "Created %d single channel spectra from file %s\r" numSpectra, S_FileName
		else
			KillWaves/Z w_sc, m_sc, wrefs_sc
			sprintf cmd, "Single channel spectrum not loaded from %s\r" S_FileName
		endif
		PrintToHistory(cmd, prefs.verbose & 2)
	endif
	
	if (prefs.loadOptions & 8) // trying to load an interferogram
		cmd = ""
		strNote  = KeypairByKey("Data Parameters [IFS]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Acquisition Parameters [SC]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("FT Parameters [SC]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Optic Parameters [SC]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Instrument Parameters [SC]", strParameters, ":", "\r")
							
		if (WaveExists(w_ifs))
			note w_ifs, strNote
			sprintf cmd "Created single channel interferogram %s from file %s\r" GetWavesDataFolder(w_ifs, 4), S_FileName
		elseif (WaveExists(m_ifs))
			note m_ifs, strNote
			sprintf cmd "Created single channel interferogram matrix %s from file %s\r" GetWavesDataFolder(m_ifs, 4), S_FileName
		elseif (WaveExists(wrefs_ifs) && numpnts(wrefs_ifs))
			numSpectra = numpnts(wrefs_ifs)
			for (i=0;i<numSpectra;i++)
				note wrefs_ifs[i], strNote
			endfor
			sprintf cmd, "Created %d single channel interferograms from file %s\r" numSpectra, S_FileName
		else
			KillWaves/Z wrefs_ifs
			sprintf cmd "Single channel interferogram not loaded from %s\r" S_FileName
		endif
		PrintToHistory(cmd, prefs.verbose & 2)
				
		cmd = ""
		strNote  = KeypairByKey("Data Parameters [IFR]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Acquisition Parameters [REF]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("FT Parameters [REF]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Optic Parameters [REF]", strParameters, ":", "\r") + "\r"
		strNote += KeypairByKey("Instrument Parameters [REF]", strParameters, ":", "\r")
				
		if (WaveExists(w_ifr))
			note w_ifr, strNote
			sprintf cmd "Created reference interferogram %s from file %s\r" GetWavesDataFolder(w_ifr, 4), S_FileName
		elseif (WaveExists(m_ifr))
			note m_ifr, strNote
			sprintf cmd "Created reference interferogram matrix %s from file %s\r" GetWavesDataFolder(m_ifr, 4), S_FileName
		elseif (WaveExists(wrefs_ifr) && numpnts(wrefs_ifr))
			numSpectra = numpnts(wrefs_ifr)
			for (i=0;i<numSpectra;i++)
				note wrefs_ifr[i], strNote
			endfor
			sprintf cmd, "Created %d reference interferograms from file %s\r" numSpectra, S_FileName
		else
			KillWaves/Z wrefs_ifr
			sprintf cmd "Reference interferogram not loaded from %s\r" S_FileName
		endif
		PrintToHistory(cmd, prefs.verbose & 2)
	endif

	PrintToHistory("\r" + strParameters, prefs.verbose & 4)
	
	if (prefs.options & 8) // display hyperspectral stack
		if (exists("ProcGlobal#Hyperspec#MakeFPAbrowser") == 6)
			if (waveexists(m_abs))
				sprintf str, "ProcGlobal#Hyperspec#MakeFPAbrowser(%s)", GetWavesDataFolder(m_abs, 4)
				execute/P/Q/Z str
			elseif (waveexists(m_tr))
				sprintf str, "ProcGlobal#Hyperspec#MakeFPAbrowser(%s)", GetWavesDataFolder(m_tr, 4)
				execute/P/Q/Z str
			elseif (waveexists(m_sc))
				sprintf str, "ProcGlobal#Hyperspec#MakeFPAbrowser(%s)", GetWavesDataFolder(m_sc, 4)
				execute/P/Q/Z str
			endif
		else
			if (waveexists(m_abs))
				Display;	AppendImage m_abs
				ModifyImage ''#0 ctabAutoscale=2
			elseif (waveexists(m_tr))
				Display;	AppendImage m_tr
				ModifyImage ''#0 ctabAutoscale=2
			elseif (waveexists(m_sc))
				Display;	AppendImage m_sc
				ModifyImage ''#0 ctabAutoscale=2
			endif
		endif
	endif
	
	
end

// turns raw jpeg binary in w_jpeg into an image wave
function/WAVE SaveImage(wave/B w_jpeg, string NewName, int overwrite)
	
	string savDF = GetDataFolder(1)
	// when waves are to be made in subfolders NewName will include the subfolder name, thus:
	// ":SingleChannelSpectra:Sample1"
	string strFolder = ""
	strFolder = ParseFilePath(0, NewName, ":", 1, 1)
	NewName = ParseFilePath(0, NewName, ":", 1, 0)
	
	if (strlen(strFolder))
		NewDataFolder/O/S $strFolder
	endif
	
	string msg = ""
	NewName = CleanupName(NewName, 0)
	if (exists(NewName))
		if (overwrite)
			if (overwrite == 2)
				sprintf msg "%s already exists. Overwrite?\r(No will rename, Cancel will skip this block)", NewName
				DoAlert 2, msg
				if (V_flag == 3)  // Cancel
					SetDataFolder savDF // restore data folder, if necessary
					return $""  // quit
				elseif (V_flag == 2) // No
					NewName = UniqueName(NewName, 1, 0) // Rename
				endif
			endif
		else
			NewName = UniqueName(NewName, 1, 0)
		endif
	endif
	
	variable refnum
	string tempPath = SpecialDirPath("Temporary", 0, 0, 0)
	Open/Z refnum as tempPath + "OPUSJPEG.jpg"
	FBinWrite refnum, w_jpeg
	Close refnum
	
	ImageLoad/Q/Z/T=jpeg/N=$(NewName)/O tempPath+"OPUSJPEG.jpg"
	wave/Z img = $StringFromList(0, S_waveNames)
	SetDataFolder savDF // restore data folder, if necessary
	return img
end

function/wave LoadDataBlock(int BlockPointer, int BlockLength, int refnum, string newName, int overwrite)
	
	string savDF = GetDataFolder(1)
	string msg
		
	// when waves are to be made in subfolders NewName will include the subfolder name, thus:
	// ":SingleChannelSpectra:Sample1"
	string strFolder = ""
	strFolder = ParseFilePath(0, NewName, ":", 1, 1)
	NewName = ParseFilePath(0, NewName, ":", 1, 0)
	
	if (strlen(strFolder))
		NewDataFolder/O/S $strFolder
	endif
	
	NewName = CleanupName(NewName, 0)
	if (exists(NewName))
		if (overwrite)
			if (overwrite == 2)
				sprintf msg "%s already exists. Overwrite?\r(No will rename, Cancel will skip this block)", NewName
				DoAlert 2, msg
				if (V_flag == 3)  // Cancel
					SetDataFolder savDF // restore data folder, if necessary
					return $""  // quit
				elseif (V_flag == 2) // No
					NewName = UniqueName(NewName, 1, 0) // Rename
				endif
			endif
		else
			NewName = UniqueName(NewName, 1, 0)
		endif
	endif
	
	Make/O/N=(BlockLength) $NewName /WAVE=w_data
	
	FStatus refNum
	int savedPosition = V_filePos
	try
		FSetPos refNum, BlockPointer; AbortOnRTE // absorbance data block
		// read the data into the new wave
		FBinRead/B=3/F=4 refnum, w_data; AbortOnRTE
	catch
		if (V_AbortCode == -4)
			Print "Error reading data block"
			int CFerror = GetRTError(1)	// 1 to clear the error
			Print GetErrMessage(CFerror)
			KillWaves/Z w_data
		endif
	endtry
	
	FSetPos refNum, savedPosition // reset position in file
	SetDataFolder savDF // restore data folder, if necessary
	return w_data // GetWavesDataFolder(w_data, 4)
end

// BlockLength is 4 times number of bytes
function/T LoadParameterBlock(int BlockPointer, int BlockLength, int refnum)
	
	try
		FStatus refNum
		int savedPosition = V_filePos
		FSetPos refNum, BlockPointer; AbortOnRTE
		
		string strList, strValue, strItem
		variable varValue
		int i, j, format, length, len
		
		strList = ""
		i = 0
		do
			strItem  = "aaaa"
			strValue = ""
			varValue = NaN
					
			FBinRead/B=3 refnum, strItem; AbortOnRTE // four bytes
			FBinRead/B=3/F=2 refnum, format; AbortOnRTE // two bytes
			FBinRead/B=3/F=2 refnum, length; AbortOnRTE // two bytes
			i += 8
			
			// length is number of 2 byte words
	
			strItem = strItem[0,2]
			if (stringmatch(strItem, "END"))
				break
			endif
			
			switch (format)
				case 0 : // integer
					if (length == 2) // seems that all integer values are 4 bytes.
						FBinRead/U/B=3/F=(3) refnum, varValue; AbortOnRTE
						i += 4
						sprintf strItem "%s=%0.20g" strItem, varValue
					else // if the field has a different length,
						// read into a string and throw away.
						strValue = PadString(strValue, 2*length, 0x20)
						FBinRead/B=3 refnum, strValue; AbortOnRTE
						i += 2*length
					endif
					break
				case 1 : // float
					if (length == 4)
						FBinRead/B=3/F=(5) refnum, varValue; AbortOnRTE
						i += 8
						sprintf strItem "%s=%0.6f" strItem, varValue
					else // if the field has a different length,
						// read into a string and throw away.
						strValue = PadString(strValue, 2*length, 0x20)
						FBinRead/B=3 refnum, strValue; AbortOnRTE
						i += 2*length
					endif
					break
				case 2 : // string
				case 3 : // string (selectable list)
				case 4 : // string (selectable list, machine dependent)
					strValue = PadString(strValue, 2*length, 0x20)
					FBinRead/B=3 refnum, strValue; AbortOnRTE
					i += 2*length
					len = strsearch(strValue, "\x00", 0)
					if (len == 0)
						strValue = ""
					elseif (len > -1)
						strValue = strValue[0,len-1]
					endif
					
					// clean up any invalid bytes - very old OPUS files may contain extended ASCII
					// that doesn't read as UTF-8
					strValue = ConvertTextEncoding(strValue , 1 , 1 , 2, 1)
					
					strItem += "=" + strValue
					break
				default : // unknown field, read into a string and throw away.
					strValue = PadString(strValue, 2*length, 0x20)
					FBinRead/B=3 refnum, strValue; AbortOnRTE
					i += 2*length
			endswitch
			
			// replace any characters that we want to use as higher level key or list separators
			strItem = ReplaceString(";", strItem, " ")
			strItem = ReplaceString("\r", strItem, " ")
			strList = AddListItem(strItem, strList, ";", Inf)
			
		while (i < (BlockLength*4-8))
	
	catch
		Print "Error reading parameter block"
		int CFerror = GetRTError(1)	// 1 to clear the error
		Print GetErrMessage(CFerror)
		strList = ""
	endtry
		
	FSetPos refNum, savedPosition
	return strList
end

// Length of block in bytes is 4 times BlockLength
function/T LoadImageParameterBlock(int BlockPointer, int BlockLength, int refnum)
	
	int offset, len
	try
		FStatus refNum
		int savedPosition = V_filePos
		FSetPos refNum, BlockPointer; AbortOnRTE
		FBinRead/U/B=3/F=(3) refnum, offset; AbortOnRTE
		FBinRead/U/B=3/F=(3) refnum, len; AbortOnRTE
		
		string strList = LoadParameterBlock(BlockPointer + offset, len/4, refnum)
		offset += len
		len = BlockLength - offset/4
		strList += LoadParameterBlock(BlockPointer + offset, len, refnum)
	catch
		Print "Error reading parameter block"
		int CFerror = GetRTError(1)	// 1 to clear the error
		Print GetErrMessage(CFerror)
		strList = ""
	endtry

	FSetPos refNum, savedPosition
	return strList
end

// returns image block as free wave
function/wave LoadImageBlock(int BlockPointer, int BlockLength, int refnum)
	
	// first load the whole block
	Make/free/B/U/N=(BlockLength*4) wImage
	try
		FStatus refNum
		int savedPosition = V_filePos
		FSetPos refNum, BlockPointer; AbortOnRTE
		FBinRead/U/B=3/F=1 refnum, wImage; AbortOnRTE
	catch
		Print "Error reading image block"
		int CFerror = GetRTError(1)	// 1 to clear the error
		Print GetErrMessage(CFerror)
	endtry
	FSetPos refNum, savedPosition
	
	// find the embedded JFIF image
	Make/free/U/B wToFind = {0xFF, 0xD8, 0xFF, 0xE0}
	FindSequence/I=wToFind wImage
	
	if (V_value > -1)
		DeletePoints 0, v_value, wImage
	else
		return $""
	endif
	
	return wImage
end

function/S LoadTextBlock(int BlockPointer, int BlockLength, int refnum)
	
	string strExtnd = ""
	try
		FStatus refNum
		int savedPosition = V_filePos
		FSetPos refNum, BlockPointer; AbortOnRTE
		strExtnd = PadString(strExtnd, 4*BlockLength, 0x20)
		FBinRead/B=3 refnum, strExtnd; AbortOnRTE
		strExtnd = ReplaceString("\x00", strExtnd, " ")
		int len
		do
			len = strlen(strExtnd)
			strExtnd = ReplaceString("  ", strExtnd, " ")
		while (len != strlen(strExtnd))
		if (strlen(strExtnd) && cmpstr(strExtnd[0], " ")==0)
			strExtnd = strExtnd[1,Inf]
		endif
	catch
		Print "Error reading extended block"
		int CFerror = GetRTError(1)	// 1 to clear the error
		Print GetErrMessage(CFerror)
		strExtnd = ""
	endtry
	FSetPos refNum, savedPosition
	return strExtnd
end

function PrintToHistory(string s, int DoIt)
	if (strlen(s)==0 || DoIt==0)
		return 0
	endif
	
	#if IgorVersion() >= 9
	printf "%s\r", RemoveEnding(s, "\r")
	#else
	int numLines = ItemsInList(s, "\r")
	int i
	string strLine = ""
	for (i=0;i<numLines;i++)
		strLine = StringFromList(i, s, "\r")
		if (strlen(strline) > 2400)
			Print "Parameter list too long to print"
		else
			printf "%s\r", RemoveEnding(strLine, "\r")
		endif
	endfor
	#endif
	return 1
end

function/S KeypairByKey(string key, string keylist, string keysep, string listsep)
	string item = StringByKey(key, keylist, keysep, listsep)
	if (strlen(item))
		return key + keysep + item
	endif
	return ""
end

// *** Data and Parameter type string functions ***

// functions to determine the block type
// the information needed to decode the type bits
// was contributed by a helpful soul.

// returns a string indicating the type of spectrum data
// based on bits 10-16 of the block type
function/S DataTypeString(int data)
	switch (DATA)
		case 1: // "spectrum, undefined Y-units"
			return "spectrum"
		case 2:
			return "interferogram"
		case 3:
			return "phase spectrum"
		case 4:
			return "absorbance spectrum"
		case 5:
			return "transmittance spectrum"
		case 6:
			return "Kubelka-Munck spectrum"
		case 7:
			return "trace (intensity over time)"
		case 8:
			return "GC file, series of interferograms"
		case 9:
			return "GC file, series of spectra"
		case 10:
			return "Raman spectrum"
		case 11:
			return "emission spectrum"
		case 12:
			return "reflectance spectrum"
		case 13:
			return "directory block"
		case 14:
			return "power spectrum (from phase calculation)"
		case 15:
			return "log reflectance (like absorbance)"
		case 16:
			return "ATR spectrum"
		case 17:
			return "photoacoustic spectrum"
		case 18:
			return "result of arithmetic, looks like TR"
		case 19:
			return "result of arithmetic, looks like AB"
	endswitch
	return ""
end

// returns a string indicating the type of parameter block
// based on bits 4-9 of the type field in the directory
function/S ParameterTypeString(int PARM)
	switch (PARM)
		case 1: // "data status parameter"
			return "Data Parameters"
		case 2: // "instrument status parameters"
			return "Instrument Parameters"
		case 3: // "standard acquisition parameters"
			return "Acquisition Parameters"
		case 4: // "FT Parameters"
			return "FT Parameters"
		case 5: // "plot and display parameters"
			return "Plot and display parameters"
		case 6: // "processing parameters"
			return "Optic Parameters"
		case 7: // "GC parameters"
			return "GC parameters"
		case 8: // "library search parameters"
			return "Library search parameters"
		case 9: // "communication parameters"
			return "Communication parameters"
		case 10: // "sample origin parameters"
			return "Sample Parameters"
	endswitch
	return ""
end

// returns a string indicating the data format
// based on bits 0 and 1 of the type field in the directory
function/S CPLXTypeString(int CPLX)
	switch (CPLX)
		case 1:
			return "real part of cplx data"
		case 2:
			return "imag part of cplx data"
		case 3:
			return "amplitude data"
	endswitch
	return ""
end

// returns a string indicating the type of spectrum
// based on bits 2 and 3 of the type field in the directory
function/S SpectrumTypeString(int STYP)
	switch (STYP)
		case 1: // "sample data"
			return "SC"
		case 2: // "reference data"
			return "REF"
		case 3: // "ratioed data"
			return "AB" // or reflectance/transmittance
	endswitch
	return ""
end

function/S DataParameterString(int STYP, int DATA)
	if (STYP==3 && DATA==4)
		return "Data Parameters [AB]"
	elseif (STYP==3 && DATA==5)
		return "Data Parameters [TR]"
	elseif (STYP==1 && DATA==1)
		return "Data Parameters [SC]"
	elseif (STYP==2 && DATA==1)
		return "Data Parameters [REF]"
	elseif (STYP==1 && DATA==2)
		return "Data Parameters [IFS]"
	elseif (STYP==2 && DATA==2)
		return "Data Parameters [IFR]"
	endif
	return ""
end

function/S ExtendedTypeString(int EXTND)
	switch (EXTND)
		case 1:
			return "Information"
		case 2:
			return "peak table"
		case 3:
			return "molecular structure"
		case 4 :
			return "macro"
		case 5:
			return "log of all actions which change data"
			
		case 11:
			return "Image parameters"
		case 20:
			return "Image Parameters from Image Block"

		case 13:
			return "History"
		case 19:
			return "Curve Fit"
		case 21:
			return "Signature"
		case 30:
			return "Integration method"
	endswitch
	return ""
end

function/S LoadOpusParameters()
			
	// open the data file
	int refnum
	string fileFilters = "All Files:.*;Data Files (*.txt,*.dat,*.csv):.txt,.dat,.csv;Weird files (*.0):.0;"
	Open/R/F=fileFilters/M="Looking for an OPUS file..." refnum
	if (!strlen(s_filename))
		return ""
	endif
	
	// extract info from header block
	int MagicNumber, version, Pointer, MaxSize, DirSize
	FSetPos refNum, 0
	FBinRead/B=2/F=3 refnum, MagicNumber
	if (MagicNumber != 0x0A0AFEFE)
		return ""
	endif
	FBinRead/B=3/F=5 refnum, version // always the same value, 920622
	FBinRead/U/B=3/F=3 refnum, Pointer
	FBinRead/U/B=3/F=3 refnum, MaxSize
	FBinRead/U/B=3/F=3 refnum, DirSize
	
	FSetPos refNum, Pointer // first directory block
	int block, BlockType, BlockLength, BlockPointer

	// read parameter blocks into strParameters as nested keylists
	string strParameters = ""
	string strItem = "", cmd = "", key = "", strExtnd = ""
	
	int CPLX, PARM, STYP, DATA, DERIV, EXTND
	
	for (block=0; block<DirSize; block+=1) // read directory info
		
		FBinRead/U/B=3/F=3 refnum, BlockType
		FBinRead/U/B=3/F=3 refnum, BlockLength
		FBinRead/U/B=3/F=3 refnum, BlockPointer
					
		// examine bits 0 and 1
		CPLX = BlockType & 3
		// examine bits 2 and 3
		STYP = (BlockType >> 2) & 3
		// examine bits 4 - 9
		PARM = (BlockType >> 4) & 0x3F
		// examine bits 10 - 16
		DATA = (BlockType >> 10) & 0x7F
		// examine bits 17 - 18
		DERIV = (BlockType >> 17) & 3
		// examine bits 19 - 25
		EXTND = (BlockType >> 19) & 0x7F
			
		// check for and load parameter block
		switch (PARM) // maybe need to exclude DERIV>0 and EXTND>0 blocks???
			case 1: // data parameters
				if (CPLX == 3) // potentially useful data parameters
					key = DataParameterString(STYP, DATA)
					if (!strlen(key))
						break
					endif
					strItem = LoadParameterBlock(BlockPointer, BlockLength, refnum)
					strParameters = ReplaceStringByKey(key, strParameters, strItem, ":", "\r")
				endif
				break
			case 2: // instrument parameters
			case 3: // acquisition parameters
			case 4: // FT parameters
			case 6: // processing parameters = optic parameters
				strItem = LoadParameterBlock(BlockPointer, BlockLength, refnum)
				key =  ParameterTypeString(PARM) + SelectString(STYP == 2, " [SC]", " [REF]")
				if (strlen(strItem))
					strParameters = ReplaceStringByKey(key, strParameters, strItem, ":", "\r")
				endif
				break
			case 10: // sample origin parameters
				strItem = LoadParameterBlock(BlockPointer, BlockLength, refnum)
				if (strlen(strItem))
					strParameters = ReplaceStringByKey("Sample Parameters", strParameters, strItem, ":", "\r")
				endif
				break
		endswitch
		
		switch (EXTND)
			case 13 :
				strExtnd = LoadTextBlock(BlockPointer, BlockLength, refnum)
				if (strlen(strExtnd))
					strParameters = ReplaceStringByKey(ExtendedTypeString(EXTND), strParameters, strExtnd, ":", "\r")
				endif
				break
//			case 11 :
//				strExtnd = LoadImageParameterBlock(BlockPointer, BlockLength, refnum)
//				break
//			case 20:
//				strExtnd = LoadImageParameterBlock(BlockPointer, BlockLength, refnum)
//				break
		endswitch
 	endfor // bottom of loop through directory
	
	Close refNum
	strParameters = SortList(strParameters, "\r")
	return RemoveEnding(strParameters, "\r")
end

// PNG: width= 90, height= 30
Picture pCog
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"&!!!!?#R18/!3BT8GQ7^D&TgHDFAm*iFE_/6AH5;7DfQssEc39jTBQ
	=U%adj95u_NKeXCo&'5*oW5;QerS2ddE(FP;'6Ql#oBiuS1dW/'TBu9uWoJd,<?@,Ku[OT_*C6>N&V
	5fgKO-Z!0m]BHZFE$kSY/b+).^UIV80KE-KEW"FQ9_,eF^bT7+UPEE3;1R;IlL+VJZGMtc/)57aYSB
	X^5kLMq7Sp9q0YBG@U214ph]9b)&eA4@(!#Ip%g0/F0_UH>\E'7M.=ui]mI6`m'G"T)B'OA[IVmQDm
	=?Eat`-Qd[5bR1c#F]`cT(Se*XgJen[DI=<Y8YF7N!d-X+iY2pq;SVDt;siWS_bs$!E]WT^pRhs]aH
	M%csq\akmJ4tMo<HGU1WqT,5cib%XrHM`>*]/&U:](RQ7QDj\u&#cMNi8>`?8;-?rCcXX>+1^gW10F
	n!h-2G-M;R\a+TD2op'?`R]"%W!DpJmO],i(@-(/F?,;L7Vkt%YOa,f\8+CV@lHVOEMgMZPnh$6>!V
	MTYBm^8f[O,?Z$2MnH6.T'JWSM4HtSissRo-):4d:ecoe5Tn^(gUEQm+o94@;L(/[91%aXk:!pP;mm
	\kh$s.7qbe%=-p1eBtDs*CHp:+$CUY\0A,jM0:4J2&pY-HWBG?nb`"BE/M-#*)+E?I*C/(r;J]APNh
	3!Ea<(u)`o?0R`ma=QV<n?GV/s3:I0Wf2_M0p@:__T%OEl+sL@10K8&ViQgR(0Q3qMLYA':/iba:,;
	]Y$@ACMV&9b[fD4A`Vq5+A!37VD0na`;0#fWNWKq#f5N>Mt)$S['[2:?=(p2$Q$$NX_cXoJ`iVOcHm
	Rb+"_b#*b4@tp)Xq9r*1_<^IVlpMJ=kE>MhiHa2]]q9<d*4(lA_8$4ej2NM5Z!#`oc=+Ttk-]%D5"O
	Yiu,o$V/I<=@2fN3Ds,PNfIEnqn6C?^[OYDs4q2k*s6TFu+@1>SKUmdko@B5>Pp)-]8`l_Ig,/1c.T
	K'Z+asa)qDc*mqZAKmijlOd;;&H$MEMWY1:\q<G#aaNVlho?TWCGL35!G658MH$RpQ,/[:S#==eP-@
	T+s%'h-7&.0)\eW6j@1gNW'FYlgRIid1g1dP0.MtL)"o@*4A+B&XU\9bRSJWg?B!keI%b6T7FS'?W(
	@7j-a-n[,Adkh%U((6"oN9G"iRV$rmb0"2lqXk08!N&V_b13Oo*tc)h=[L]E@W<ihr:]%Dbs-*cCbc
	T^`<b81D1(d_gue7JX)rMl-Q!ag!0.a4mbCL5'MQH2X<p3`1nA#69QNiWDnN[J^:kIm`JPCXo#W8lo
	?KFN_dOMp#7s\i!;q:1Vb`q^Za5j'0Sg8ALVn\tm:OM*.G/IF;=K4S+O/0U]^`u\O,i'69Y*0'f,SH
	:Kp)mH3.EQ3hDf%?f;W[LbJbR7R5Lb$8U4I7,[8ZM7fU7H5(>6BlSlr1cG6"263q!"YH.\`>aLYN(!
	e3hZYm666$Mq_c(_GHO?%CE(rnI-UV=I6M\e$%CXt$`9q"IB8d`/4e)0&Dcf`43oobf6MqdV?d>\73
	X/dI(2jZ+#[Nt'sQg5KRFl_^rEVX>cZ5h3g1gR#*IhTZf+KqmsVrW[`UcE6>MM*N0fMcT[S!:h'=ju
	EaL@5>OfU]WXd+d/JISLYCQZ_BPkB$IiAYUBq1l^ecC4a8EYJ'WJ,pak5V59k6$F23m\(d</D&W$.c
	&:n64N(\`j0%h;m7iB=j)(R^kh8BUCfn<.JP[26K1F7\>0JOc56jWCOX(6k=i$m^+A%<G5QZ.i$e8]
	01cU/k$R?&@c[1P\L>tK[OkSMmQ7lT_puI&4&#-'R99q+p;3\Sn`Ic2G%kDj/"N6oB<E0?Z6Kl"@,Y
	?4P5G,Nu\q^LTs(Vj(g3YCoZEl;W9!T)>ePG,S!10n_Y\rP(FBr90.J3;jVP^MYp<-ML6>(=)*bFsB
	&uWX-VWg'<P8G@$+\A?J1@G'F,/Z.<q/;.,=8I?;gFp>>;IjEQPE_:7`.R+3bElA@DB6<k@ksJ9lg(
	=CVM=g<G(^E#SiiFHZ8.qF-^ppkE&\[U*_);<'Lfk*Fq]^#W339=Q'IWpj"KgUWU]1,ETW<&G^OtOH
	MkGSX1r0@A!=Gd&>m2;$s(nF8b!K?8R`eZjO8V38_I$<1AcnP!)B*`T#3.X=hE[s8PMoFf*/H2@32u
	N'Len`GUk?n:K*@=9gMMi1ES9gF7flQcCD`2n^,h:`S5=GNQ^G#@^/a:?]W`PV50n4Xue>QVk8E1=]
	lWKB?uV(SiXjL_hVC,G-+W-bHd)[C`^u(BPM:VV58ltJcZ8d$CEhp-Ek)Qb.'^'f)4eT`-]7*:O[1>
	WO?>HRRZ3%&Aum4gNS.jU=.^S*B#H\'0H3Z)tG>O;)WW1_IO09[>@PI4Y3R7J0]^!UgQhj+u1-,O_#
	4q7aj2?Y5.V]ph=D*J`g8?n%JH:8P)K%MLrfVTs(X1]A:d+mFtdNBG"";'8siHNZC4&bKJq`%mNb7r
	SW;=`2-+n=L)HDOsFHoS$CX_6m<3W758iRSt7"9?7u`s%5^"&NsfV]&fKiR),nod"F>!-ZZj257RH'
	BCpkU2>pgE:BYW?=m,Fa:(R[)N0g+8U_d1?h6?87.J?1.S8QM+N.?c/5H^[K9Qq/KSe*09PFi*)kOs
	Cpa8151hB![K\`b9:/2t#`0h)TQGGW^_mOCaj@jCA@uU*q95,uIW@7!X&<O\"P/_=YGg"!\PP(fcMs
	XX8]B01I/5GVm/oHal.qLkA^e?r`*cq5<6cMYpEN]i/%/Vl(p'^fIMd8C\oHLnT1!)7oQm]mKeID*1
	Nn<=:2#ZkEe#YpHHH,[1j)T%7I4;@/)4cu_Q1)PbZM:[@i"UEI%;r>p03XoVZ_RkU=+$'7#=USG/bL
	8-+m<=>h,iqN=A8kNQ3E0-ce+TlUO7L$\:&5CW07\^Y5(=Lpj3bn5fXf]+hD?H]7Wq?&[-c"7hNK0#
	/)B'Mj<HT8ma3CurC,*bh\'aHR7Z[QbX-ZmAk800m):lpO:?P+(!;TY'R\j"AmjWGZ^'4n^bf?W3J@
	>&NBK1HqWkVhSJ@2:%U$9,hq:d,G1cCmI-TcrP\J*Ut[2@6?BcK3XN6]^DH?sm>]m;PWk0+t]M3*pb
	_i5ToaNr1&dko4ib1O7G-^#a3R58IWd+6c;6ULrU<E06&]A7B"H::^+p=jM"Cht@E-\k9W-F%RN7[f
	g9`s&hll-^kfa*7KZfMd!Yc("^(?tb@(G_h8BF>IEA!<RhTlND,!db&r!Y3m>2$M&6dHhJmnR;"(TN
	7k9deNFM:rt_%M:_\bI5MO2h<A0N#R:ZSrC"&ps\iY*%&:=-;@IrX+"G9!l_&sOI?=_'7)$hsk)[Og
	CfZ9V$@mNB]AS#G_>V6^Z_/)$iF?19\*_+U8'Lh!@O$@74\ogtP<K2K,l#,B;0e6NJ*#oS35C1Gqa?
	[/#EMYbdp\'g083t^Hm.OC']=?TCh[+\@'/CD^$lb9k>s0TnKIaqh4fI!&lDq*\4*U*,*??/2AnId;
	.P@%q^Y_gV7L#<Y@CP!Nm,EKn=&i8<r@!PTa5]H_PQ](fBqn;iMUE,Pl5E-<#!*W9G0Gifi6\]*<o4
	FnfrX,WF^_\F$1nF]^fU-bLO!=bm%5lG.k3$IWMqUu'c@l,R*B4I#7$5RGsBAo;S"q!W&r%8C2/"PK
	bsa<4<J73edT0;DW#W4)GM@uFDONL"9R+_N^([T!(k&'aJc*VLHV&^R;!(_#4Z$g7@#[rnEd4b][m6
	"s2Cf6FX#Yth+![-Bc9;DCc35!#ZOe]>R5l%A3s9r*"E3cZ^HAq!@!X3ZLR*.A7oQ8om.\p[kV)RYJ
	YS_VJ,ke(!91A):`eg`H3<A0s%C//=2RBq,pC2:S]*\P@U_Oa*3.T[g!Hf.uI$^Z574:Ig+a&Rh&b)
	g:i!IBPVCY]Y$?Mc-nM/==coe'#A=jP*M;$C2,4.LP+[KAA[:ZiG]VW6ipmf;5gRtUogbYmG#,M=Y+
	23\;kLm>:;.qMlKqn)uClJ![.i',6Vo?VRu"<5C39QHia.rIUXNcq/492/E<t4:q=5j]#0#BT^2C8R
	r=7NOaA&EGCimE'I"(o(b72sE7cRPmWBnj]tHh/;(=(Hs(iTH$*GN/REC6P4"-OQ)1Z*S9OlNXqYG,
	/qFo"%Xt5WcH)DMDo_@'gn2mp\l)\(b!Y0Pa!2n,Nj%-R@Yjn?WT$E#t(FUbj=.R08ON,:0qYL%:/M
	0]<Q1)2)G':0@s*h8ZZ<4MLPu4'A3d&SI6l9j+cCI%0ltDh?G(&13<0N\Q=?t??;p9Q8$59_n3Rli=
	5b`N"1`$#>;[JNr+$!*^fli%qGlHBaGd#r_gndbH0<a<pR0sE5L0:j)I_t)\EH/7Wqt8QJMd<r<&WK
	8J3cuoH9hij#22_bS-?/1q+bUC@(DjDc_1Dg*LCYK([C$_m"OB=44C54XF6CiRHM)#JSik-Qi#lgdX
	C:AAV;n0(-%L_0m!T,"d5MVG@HhTKZ87K.p%WH^Xh3kCpXcTY<@p]%p!H!PcGm8Ma`dWI-i:%O`.@A
	\EMhGlp>Rk7O<G2m`*r,h[u\8;4r,bUaQp%EDN*J]D4B1hFXuppq^tpModARV5%<QlNN?L%hAEkIlW
	/#`^]Bs#-d.f-97RH2#l<o@ZXZ&T?iRH-<%N9SU+'so7AAgW2mil0fWaM)A,6@)4Rp@WFC0@Y,uIN:
	5uCJkMPAJFd6VVd/UR6[rI=?0fVgq/kI,s^(YARDN54WD\PD#"bV<C5XKS<`]Fql#m@)ul]O#Nn]-2
	[lB);<HM,sPARkJs:;d4_S!/nh?qGe7@I:AsnMi7DjM_D$2XW>fsY^ZQI8$;`nm!f"mg^^mq'BNs/!
	!!!j78?7R6=>B
	ASCII85End
end


function/wave LoadMatrixBlock(int BlockPointer, int BlockLength, int refnum, string newName, int overwrite, int splitmatrix)
	
	string savDF = GetDataFolder(1)
	string msg
		
	// when waves are to be made in subfolders NewName will include the subfolder name, thus:
	// ":SingleChannelSpectra:Sample1"
	string strFolder = ""
	strFolder = ParseFilePath(0, NewName, ":", 1, 1)
	NewName = ParseFilePath(0, NewName, ":", 1, 0)
	
	if (strlen(strFolder))
		NewDataFolder/O/S $strFolder
	endif

	NewName = CleanupName(NewName, 0)
	if (splitmatrix==0 && exists(NewName))
		if (overwrite)
			if (overwrite == 2)
				sprintf msg "%s already exists. Overwrite?\r(No will rename, Cancel will skip this block)", NewName
				DoAlert 2, msg
				if (V_flag == 3)  // Cancel
					SetDataFolder savDF // restore data folder, if necessary
					return $""  // quit
				elseif (V_flag == 2) // No
					NewName = UniqueName(NewName, 1, 0) // Rename
				endif
			endif
		else
			NewName = UniqueName(NewName, 1, 0)
		endif
	endif
	
	if (splitmatrix && overwrite != 1)
		if (strlen(GrepList(WaveList(NewName + "_*", ";", ""), "^"+NewName+"_\d+$")))
			if (overwrite == 0)
				DoAlert 0, "Cannot overwrite existing data with names like " + NewName + "_001"
				SetDataFolder savDF // restore data folder, if necessary
				return $""
			elseif (overwrite == 2)
				DoAlert 1, "Overwrite existing data with names like " + NewName + "_001?"
				if (v_flag == 2)
					SetDataFolder savDF // restore data folder, if necessary
					return $""
				endif
			endif
		endif
	endif
	
	FStatus refNum
	int savedPosition = V_filePos
	
	int version, numBlocks, offset, blocksize, infosize, numentries, stringsize
	int i, digits
	string strName
		
	int nss // actual no of scans
	int nsr // actual no of scans reference
	int run // actual no of runs
	int npt // number of points
	int numGoodFW // number of good forward scans
	int numGoodBW // number of good backward scans
	int numBadFW // number of bad forward scans
	int numBadBW // number of bad backward scans
	variable hfl // high folding limit
	variable lfl // low folding limit
	variable hffl // high folding limit after filtering
	variable lffl // low folding limit after filtering
	int FilterSize // Number of filter coef.
	int FilterType // Type of filter
	variable ffp // freq of 1st point
	variable flp // freq of last point
	variable vmin // minimum of array
	variable vmax // maximum of array
	variable scf // scaling factor
	variable pka_fw // peak amplitude forward part
	variable pka_bw // peak amplitude bw part
	int pkl_fw // peak location forw part
	int pkl_bw // peak loc bw part
	variable start_time // in sec
	variable end_time // in sec

	try
		FSetPos refNum, BlockPointer; AbortOnRTE
		// read the data into the new wave
		FBinRead/U/B=3/F=3 refnum, version; AbortOnRTE
		FBinRead/U/B=3/F=3 refnum, numBlocks; AbortOnRTE
		FBinRead/U/B=3/F=3 refnum, offset; AbortOnRTE
		FBinRead/U/B=3/F=3 refnum, blocksize; AbortOnRTE
		FBinRead/U/B=3/F=3 refnum, infosize; AbortOnRTE
		FBinRead/U/B=3/F=3 refnum, numentries; AbortOnRTE
		Make/free/I/U/N=(2*numentries) wTable
		FBinRead/U/B=3/F=3 refnum, wTable; AbortOnRTE

		stringsize = infosize - 152
		digits = 1+floor(log(numBlocks))
		string strNote = "", info = ""

		FSetPos refNum, BlockPointer+offset; AbortOnRTE
		
		variable edge = sqrt(numBlocks)
		
		if (!splitmatrix)
			Make/O/N=(edge, edge, blocksize/4) $NewName /wave=Mat
			Make/free/N=(blocksize/4) w
		endif
		
		Make/free/N=0/WAVE refs
				
		for (i=0;i<numblocks;i++)

			if (splitmatrix)

				sprintf strName, "%s_%0*d", NewName, digits, i
				Make/O/N=(blocksize/4) $strName /wave=w // testing with /O
				FBinRead/B=3/F=4 refnum, w; AbortOnRTE

				FBinRead/U/B=3/F=3 refnum, nss; AbortOnRTE
				FBinRead/U/B=3/F=3 refnum, nsr; AbortOnRTE
				FBinRead/U/B=3/F=3 refnum, run; AbortOnRTE
				FBinRead/U/B=3/F=3 refnum, npt; AbortOnRTE
				FBinRead/U/B=3/F=3 refnum, numGoodFW; AbortOnRTE
				FBinRead/U/B=3/F=3 refnum, numGoodBW; AbortOnRTE
				FBinRead/U/B=3/F=3 refnum, numBadFW // number of bad forward scans
				FBinRead/U/B=3/F=3 refnum, numBadBW // number of bad backward scans
				FBinRead/B=3/F=5 refnum, hfl // high folding limit
				FBinRead/B=3/F=5 refnum, lfl // low folding limit
				FBinRead/B=3/F=5 refnum, hffl // high folding limit after filtering
				FBinRead/B=3/F=5 refnum, lffl // low folding limit after filtering
				FBinRead/U/B=3/F=3 refnum, FilterSize // Number of filter coef.
				FBinRead/U/B=3/F=3 refnum, FilterType // Type of filter
				FBinRead/B=3/F=5 refnum, ffp; AbortOnRTE
				FBinRead/B=3/F=5 refnum, flp; AbortOnRTE
				FBinRead/B=3/F=5 refnum, vmin // minimum of array
				FBinRead/B=3/F=5 refnum, vmax // maximum of array
				FBinRead/B=3/F=5 refnum, scf // scaling factor
				FBinRead/B=3/F=5 refnum, pka_fw // peak amplitude forward part
				FBinRead/B=3/F=5 refnum, pka_bw // peak amplitude bw part
				FBinRead/U/B=3/F=3 refnum, pkl_fw // peak location forw part
				FBinRead/U/B=3/F=3 refnum, pkl_bw // peak loc bw part
				FBinRead/B=3/F=5 refnum, start_time // in sec
				FBinRead/B=3/F=5 refnum, end_time
				sprintf strNote, "NSS=%g;NSR=%g;RUN=%g;NPT=%g;numGoodFW=%g;numGoodBW=%g;numBadFW=%g;", nss, nsr, run, npt, numGoodFW, numGoodBW, numBadFW
				sprintf strNote, "%snumBadBW=%g;HFL=%g;LFL=%g;HFFL=%g;LFFL=%g;FilterSize=%g;FilterType=%g;", strNote, numBadBW , hfl , lfl, hffl, lffl, FilterSize, FilterType
				sprintf strNote, "%sFFP=%g;FLP=%g;VMIN=%g;VMAX=%g;SCF=%g;PKA_FW=%g;PKA_BW=%g;", strNote, ffp, flp, vmin, vmax, scf, pka_fw, pka_bw
				sprintf strNote, "%sPKL_FW=%g;PKL_BW=%g;StartTime=%g;EndTime=%g;", strNote, pkl_fw, pkl_bw, start_time, end_time
				note w, strNote
				if (stringsize)
					info = PadString(info, stringsize, 0)
					FBinRead refnum, info
					note w, info
				endif
				
				refs[i] = {w}

			else
				FSetPos refNum, BlockPointer+offset+i*(blocksize+infosize); AbortOnRTE
				FBinRead/B=3/F=4 refnum, w; AbortOnRTE
				Mat[mod(i, edge)][floor(i/edge)][] = w[r] // integer math makes i/edge the same as floor(i/edge) ?
				
			endif

		endfor
	
	catch
		if (V_AbortCode == -4)
			Print "Error reading data block"
			int CFerror = GetRTError(1)	// 1 to clear the error
			Print GetErrMessage(CFerror)
		endif
	endtry
	
	FSetPos refNum, savedPosition // reset position in file
	SetDataFolder savDF // restore data folder, if necessary
	
	if (!splitmatrix)
		return Mat
	else
		return refs
	endif
		
end

function AddToIgorProcedures(int add)
	string thisFile = FunctionPath("")
	string IgorProcs = SpecialDirPath("Igor Pro User Files", 0, 0, 0) + "Igor Procedures:"
	
	if (stringmatch(thisFile, IgorProcs + "*"))
		// this file is in Igor Procdures folder
		if (!add)
			MoveFile/D thisFile as SpecialDirPath("Igor Pro User Files", 0, 0, 0) + "User Procedures:"
		endif
		return 0
	endif
	
	string theShortcut = "", ext = "alis", strShortcut = "alias"
	
	#ifdef WINDOWS
	ext = ".lnk"
	strShortcut = "shortcut"
	#endif
	
	// get a list of shortcuts/aliases
	NewPath/O/Q/Z tempPathACW, IgorProcs
	wave/T w = ListToTextWave(IndexedFile(tempPathACW, -1, ext), ";")
	KillPath/Z tempPathACW
	
	// check for an existing alias in Igor Procedures folder
	int i
	int numShortcuts = numpnts(w)
	string aliasedFile = ""
	for (i=0;i<numShortcuts;i++)
		aliasedFile = ResolveAlias(IgorProcs + w[i])
		if (cmpstr(aliasedFile, thisFile) == 0)
			theShortcut = IgorProcs + w[i]
			break
		endif
	endfor
	
	if (strlen(aliasedFile) && !add)
		DeleteFile/Z theShortcut
	endif
	
	if (add && strlen(aliasedFile)==0)
		CreateAliasShortcut thisFile as IgorProcs + "OPUSloader_" + strShortcut
	endif
	
	return 0
end

// Recursively resolve shortcuts to files/directories
// returns full path or an empty string if the file does not exist or the
// shortcut points to a non existing file/folder
function/S ResolveAlias(string strPath) // full path to file, folder or alias
	GetFileFolderInfo/Q/Z RemoveEnding(strPath, ":")
	if (V_isAliasShortcut)
		return ResolveAlias(S_aliasPath)
	endif
	if (v_flag == 0)
		return strPath // path to a file or folder
	endif
	return ""
end